@@ -108,7 +108,20 @@ function build_octavia_worker_image {
         fi
         sudo mkdir -m755 ${dib_logs}
         sudo chown $STACK_USER ${dib_logs}
-        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -l ${dib_logs}/$(basename $OCTAVIA_AMP_IMAGE_FILE).log $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
+        # Build amphora image with master DIB in a Python 3 virtual environment
+        (
+            DIB_VENV_DIR=$(mktemp -d)
+            DIB_GIT_DIR=/tmp/dib-octavia
+            python3 -m venv $DIB_VENV_DIR
+            export USE_PYTHON3=True
+            export PATH=$DIB_VENV_DIR/bin:$PATH
+            if ! [ -d $DIB_GIT_DIR ]; then
+                git clone ${GITREPO["diskimage-builder"]} $DIB_GIT_DIR
+            fi
+            (cd $REQUIREMENTS_DIR && git show origin/master:upper-constraints.txt) | sed '/diskimage-builder/d' > $DIB_VENV_DIR/u-c.txt
+            pip install -c $DIB_VENV_DIR/u-c.txt $DIB_GIT_DIR
+            $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -l ${dib_logs}/$(basename $OCTAVIA_AMP_IMAGE_FILE).log $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
+        )
     fi
     upload_image file://${OCTAVIA_AMP_IMAGE_FILE} $TOKEN
 
@@ -208,20 +221,14 @@ function create_octavia_accounts {
     if [[ "$WSGI_MODE" == "uwsgi" ]] && [[ "$OCTAVIA_NODE" == "main" ]] ; then
         get_or_create_endpoint $octavia_service \
             "$REGION_NAME" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/$OCTAVIA_SERVICE_TYPE" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/$OCTAVIA_SERVICE_TYPE" \
             "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/$OCTAVIA_SERVICE_TYPE"
     elif [[ "$WSGI_MODE" == "uwsgi" ]]; then
         get_or_create_endpoint $octavia_service \
             "$REGION_NAME" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE" \
             "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE"
     else
         get_or_create_endpoint $octavia_service \
             "$REGION_NAME" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
             "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
     fi
 }
@@ -720,8 +727,6 @@ function octavia_init {
         create_octavia_accounts
 
         add_load-balancer_roles
-
-        configure_rsyslog
     elif [ $OCTAVIA_NODE == 'api' ] ; then
         create_octavia_accounts
 
@@ -732,6 +737,7 @@ function octavia_init {
         create_mgmt_network_interface
         create_amphora_flavor
         configure_lb_mgmt_sec_grp
+        configure_rsyslog
     fi
 
     if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
