@@ -24,6 +24,7 @@ from stevedore import driver as stevedore_driver
 from octavia_lib.api.drivers import data_models as driver_dm
 from octavia_lib.api.drivers import exceptions
 from octavia_lib.api.drivers import provider_base as driver_base
+from octavia_lib.common import constants as lib_consts
 
 from octavia.api.drivers.amphora_driver import availability_zone_schema
 from octavia.api.drivers.amphora_driver import flavor_schema
@@ -71,6 +72,13 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                                           project_id=project_id, vip=vip_obj)
 
         network_driver = utils.get_network_driver()
+        vip_network = network_driver.get_network(
+            vip_dictionary[lib_consts.VIP_NETWORK_ID])
+        if not vip_network.port_security_enabled:
+            message = "Port security must be enabled on the VIP network."
+            raise exceptions.DriverError(user_fault_string=message,
+                                         operator_fault_string=message)
+
         try:
             vip = network_driver.allocate_vip(lb_obj)
         except network_base.AllocateVIPException as e:
@@ -123,24 +131,37 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                    consts.LOAD_BALANCER_UPDATES: lb_dict}
         self.client.cast({}, 'update_load_balancer', **payload)
 
+    def _encrypt_tls_container_data(self, tls_container_data):
+        for key, val in tls_container_data.items():
+            if isinstance(val, bytes):
+                tls_container_data[key] = self.fernet.encrypt(val)
+            elif isinstance(val, list):
+                encrypt_vals = []
+                for i in val:
+                    if isinstance(i, bytes):
+                        encrypt_vals.append(self.fernet.encrypt(i))
+                    else:
+                        encrypt_vals.append(i)
+                tls_container_data[key] = encrypt_vals
+
     def _encrypt_listener_dict(self, listener_dict):
         # We need to encrypt the user cert/key data for sending it
         # over messaging.
         if listener_dict.get(consts.DEFAULT_TLS_CONTAINER_DATA, False):
-            listener_dict[consts.DEFAULT_TLS_CONTAINER_DATA] = (
-                self.fernet.encrypt(
-                    listener_dict[consts.DEFAULT_TLS_CONTAINER_DATA]))
+            container_data = listener_dict[consts.DEFAULT_TLS_CONTAINER_DATA]
+            self._encrypt_tls_container_data(container_data)
         if listener_dict.get(consts.SNI_CONTAINER_DATA, False):
             sni_list = []
             for sni_data in listener_dict[consts.SNI_CONTAINER_DATA]:
-                sni_list.append(self.fernet.encrypt(sni_data))
+                self._encrypt_tls_container_data(sni_data)
+                sni_list.append(sni_data)
             if sni_list:
                 listener_dict[consts.SNI_CONTAINER_DATA] = sni_list
 
     # Listener
     def listener_create(self, listener):
         payload = {consts.LISTENER: listener.to_dict()}
-        self._encrypt_listener_dict(payload)
+        self._encrypt_listener_dict(payload[consts.LISTENER])
 
         self.client.cast({}, 'create_listener', **payload)
 
@@ -165,16 +186,16 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         if 'admin_state_up' in pool_dict:
             pool_dict['enabled'] = pool_dict.pop('admin_state_up')
         if 'tls_container_ref' in pool_dict:
-            pool_dict['tls_container_id'] = pool_dict.pop('tls_container_ref')
+            pool_dict['tls_certificate_id'] = pool_dict.pop(
+                'tls_container_ref')
         pool_dict.pop('tls_container_data', None)
         if 'ca_tls_container_ref' in pool_dict:
             pool_dict['ca_tls_certificate_id'] = pool_dict.pop(
                 'ca_tls_container_ref')
         pool_dict.pop('ca_tls_container_data', None)
-        if 'client_crl_container_ref' in pool_dict:
-            pool_dict['client_crl_container_id'] = pool_dict.pop(
-                'client_crl_container_ref')
-        pool_dict.pop('client_crl_container_data', None)
+        if 'crl_container_ref' in pool_dict:
+            pool_dict['crl_container_id'] = pool_dict.pop('crl_container_ref')
+        pool_dict.pop('crl_container_data', None)
         return pool_dict
 
     def pool_create(self, pool):
