@@ -104,7 +104,7 @@ class ControllerWorker(object):
 
             return job_id
         except Exception as e:
-            LOG.error('Failed to create an amphora due to: {}'.format(str(e)))
+            LOG.error('Failed to create an amphora due to: %s', str(e))
 
     def delete_amphora(self, amphora_id):
         """Deletes an existing Amphora.
@@ -139,6 +139,12 @@ class ControllerWorker(object):
             db_apis.get_session(),
             id=health_monitor[constants.HEALTHMONITOR_ID])
 
+        if not db_health_monitor:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'healthmonitor',
+                        health_monitor[constants.HEALTHMONITOR_ID])
+            raise db_exceptions.NoResultFound
+
         pool = db_health_monitor.pool
         pool.health_monitor = db_health_monitor
         load_balancer = pool.load_balancer
@@ -1047,7 +1053,7 @@ class ControllerWorker(object):
         except Exception as e:
             with excutils.save_and_reraise_exception(reraise=False):
                 LOG.exception("LB %(lbid)s failover exception: %(exc)s",
-                              {'lbid': load_balancer_id, 'exc': e})
+                              {'lbid': load_balancer_id, 'exc': str(e)})
                 self._lb_repo.update(
                     db_apis.get_session(), load_balancer_id,
                     provisioning_status=constants.ERROR)
@@ -1062,7 +1068,8 @@ class ControllerWorker(object):
 
         amp = self._amphora_repo.get(db_apis.get_session(),
                                      id=amphora_id)
-        LOG.info("Start amphora cert rotation, amphora's id is: %s", amp.id)
+        LOG.info("Start amphora cert rotation, amphora's id is: %s",
+                 amphora_id)
 
         store = {constants.AMPHORA: amp.to_dict(),
                  constants.AMPHORA_ID: amphora_id}
@@ -1070,6 +1077,8 @@ class ControllerWorker(object):
         self.services_controller.run_poster(
             flow_utils.cert_rotate_amphora_flow,
             store=store)
+        LOG.info("Finished amphora cert rotation, amphora's id was: %s",
+                 amphora_id)
 
     def update_amphora_agent_config(self, amphora_id):
         """Update the amphora agent configuration.
@@ -1097,3 +1106,5 @@ class ControllerWorker(object):
         self.services_controller.run_poster(
             flow_utils.update_amphora_config_flow,
             store=store)
+        LOG.info("Finished amphora agent configuration update, amphora's id "
+                 "was: %s", amphora_id)
