@@ -27,7 +27,6 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import uuidutils
-from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import noload
 from sqlalchemy.orm import subqueryload
 from sqlalchemy.sql.expression import false
@@ -107,7 +106,7 @@ class BaseRepository(object):
         """
         with session.begin(subtransactions=True):
             tags = model_kwargs.pop('tags', None)
-            if tags:
+            if tags is not None:
                 resource = session.query(self.model_class).get(id)
                 resource.tags = tags
             session.query(self.model_class).filter_by(
@@ -1064,12 +1063,15 @@ class ListenerRepository(BaseRepository):
 
     def update(self, session, id, **model_kwargs):
         with session.begin(subtransactions=True):
-            tags = model_kwargs.pop('tags', None)
-            if tags:
-                resource = session.query(self.model_class).get(id)
-                resource.tags = tags
             listener_db = session.query(self.model_class).filter_by(
                 id=id).first()
+            if not listener_db:
+                raise exceptions.NotFound(
+                    resource=data_models.Listener._name(), id=id)
+            tags = model_kwargs.pop('tags', None)
+            if tags is not None:
+                resource = session.query(self.model_class).get(id)
+                resource.tags = tags
             # Verify any newly specified default_pool_id exists
             default_pool_id = model_kwargs.get('default_pool_id')
             if default_pool_id:
@@ -1247,10 +1249,7 @@ class AmphoraRepository(BaseRepository):
                 .filter(models.Amphora.status != consts.DELETED)
                 # And the LB is also not DELETED
                 .filter(models.LoadBalancer.provisioning_status !=
-                        consts.DELETED)
-                # And what does this do? Some SQLAlchemy magic?
-                .options(joinedload('*'))
-            ).first()
+                        consts.DELETED)).first()
             if db_lb:
                 return db_lb.to_data_model()
             return None
