@@ -11,12 +11,11 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-import mock
-
-from oslo_utils import uuidutils
+from unittest import mock
 
 from octavia_lib.api.drivers import data_models as driver_dm
 from octavia_lib.api.drivers import exceptions
+from oslo_utils import uuidutils
 
 from octavia.api.drivers.amphora_driver.v1 import driver
 from octavia.common import constants as consts
@@ -77,7 +76,8 @@ class TestAmphoraDriver(base.TestRpc):
             loadbalancer_id=self.sample_data.lb_id)
         self.amp_driver.loadbalancer_create(provider_lb)
         payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id,
-                   consts.FLAVOR: None}
+                   consts.FLAVOR: None,
+                   consts.AVAILABILITY_ZONE: None}
         mock_cast.assert_called_with({}, 'create_load_balancer', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -372,7 +372,8 @@ class TestAmphoraDriver(base.TestRpc):
                            'protocol_port': 80,
                            'pool_id': self.sample_data.pool1_id}
 
-        self.amp_driver.member_batch_update(prov_members)
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, prov_members)
 
         payload = {'old_member_ids': [self.sample_data.member1_id],
                    'new_member_ids': [self.sample_data.member3_id],
@@ -406,13 +407,27 @@ class TestAmphoraDriver(base.TestRpc):
                            'protocol_port': 80,
                            'pool_id': self.sample_data.pool1_id}
 
-        self.amp_driver.member_batch_update(prov_members)
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, prov_members)
 
         payload = {'old_member_ids': [self.sample_data.member1_id],
                    'new_member_ids': [self.sample_data.member3_id],
                    'updated_members': [update_mem_dict]}
         mock_cast.assert_called_with({}, 'batch_update_members', **payload)
 
+    @mock.patch('octavia.db.api.get_session')
+    @mock.patch('octavia.db.repositories.PoolRepository.get')
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_member_batch_update_clear_already_empty(
+            self, mock_cast, mock_pool_get, mock_session):
+        mock_pool = mock.MagicMock()
+        mock_pool_get.return_value = mock_pool
+
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, [])
+
+        mock_cast.assert_not_called()
+
     # Health Monitor
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_health_monitor_create(self, mock_cast):
@@ -467,7 +482,8 @@ class TestAmphoraDriver(base.TestRpc):
                            'protocol_port': 80,
                            'pool_id': self.sample_data.pool1_id}
 
-        self.amp_driver.member_batch_update(prov_members)
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, prov_members)
 
         payload = {'old_member_ids': [self.sample_data.member1_id],
                    'new_member_ids': [self.sample_data.member3_id],
@@ -505,7 +521,7 @@ class TestAmphoraDriver(base.TestRpc):
 
         self.assertRaises(exceptions.UnsupportedOptionError,
                           self.amp_driver.member_batch_update,
-                          prov_members)
+                          self.sample_data.pool1_id, prov_members)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_health_monitor_update(self, mock_cast):
@@ -658,3 +674,45 @@ class TestAmphoraDriver(base.TestRpc):
                         'SUPPORTED_FLAVOR_SCHEMA', 'bogus'):
             self.assertRaises(exceptions.DriverError,
                               self.amp_driver.validate_flavor, 'bogus')
+
+    # Availability Zone
+    def test_get_supported_availability_zone_metadata(self):
+        test_schema = {
+            "properties": {
+                "test_name": {"description": "Test description"},
+                "test_name2": {"description": "Another description"}}}
+        ref_dict = {"test_name": "Test description",
+                    "test_name2": "Another description"}
+
+        # mock out the supported_availability_zone_metadata
+        with mock.patch('octavia.api.drivers.amphora_driver.'
+                        'availability_zone_schema.'
+                        'SUPPORTED_AVAILABILITY_ZONE_SCHEMA', test_schema):
+            result = self.amp_driver.get_supported_availability_zone_metadata()
+        self.assertEqual(ref_dict, result)
+
+        # Test for bad schema
+        with mock.patch('octavia.api.drivers.amphora_driver.'
+                        'availability_zone_schema.'
+                        'SUPPORTED_AVAILABILITY_ZONE_SCHEMA', 'bogus'):
+            self.assertRaises(
+                exceptions.DriverError,
+                self.amp_driver.get_supported_availability_zone_metadata)
+
+    def test_validate_availability_zone(self):
+        ref_dict = {consts.COMPUTE_ZONE: 'my_compute_zone'}
+        self.amp_driver.validate_availability_zone(ref_dict)
+
+        # Test bad availability zone metadata key
+        ref_dict = {'bogus': 'bogus'}
+        self.assertRaises(exceptions.UnsupportedOptionError,
+                          self.amp_driver.validate_availability_zone,
+                          ref_dict)
+
+        # Test for bad schema
+        with mock.patch('octavia.api.drivers.amphora_driver.'
+                        'availability_zone_schema.'
+                        'SUPPORTED_AVAILABILITY_ZONE_SCHEMA', 'bogus'):
+            self.assertRaises(exceptions.DriverError,
+                              self.amp_driver.validate_availability_zone,
+                              'bogus')
