@@ -11,12 +11,11 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-import mock
-
-from oslo_utils import uuidutils
+from unittest import mock
 
 from octavia_lib.api.drivers import data_models as driver_dm
 from octavia_lib.api.drivers import exceptions
+from oslo_utils import uuidutils
 
 from octavia.api.drivers.amphora_driver.v2 import driver
 from octavia.common import constants as consts
@@ -76,8 +75,9 @@ class TestAmphoraDriver(base.TestRpc):
         provider_lb = driver_dm.LoadBalancer(
             loadbalancer_id=self.sample_data.lb_id)
         self.amp_driver.loadbalancer_create(provider_lb)
-        payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id,
-                   consts.FLAVOR: None}
+        payload = {consts.LOADBALANCER: provider_lb.to_dict(),
+                   consts.FLAVOR: None,
+                   consts.AVAILABILITY_ZONE: None}
         mock_cast.assert_called_with({}, 'create_load_balancer', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -85,7 +85,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_lb = driver_dm.LoadBalancer(
             loadbalancer_id=self.sample_data.lb_id)
         self.amp_driver.loadbalancer_delete(provider_lb)
-        payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id,
+        payload = {consts.LOADBALANCER: provider_lb.to_dict(),
                    'cascade': False}
         mock_cast.assert_called_with({}, 'delete_load_balancer', **payload)
 
@@ -103,7 +103,7 @@ class TestAmphoraDriver(base.TestRpc):
             loadbalancer_id=self.sample_data.lb_id, admin_state_up=True)
         lb_dict = {'enabled': True}
         self.amp_driver.loadbalancer_update(old_provider_lb, provider_lb)
-        payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id,
+        payload = {consts.ORIGINAL_LOADBALANCER: old_provider_lb.to_dict(),
                    consts.LOAD_BALANCER_UPDATES: lb_dict}
         mock_cast.assert_called_with({}, 'update_load_balancer', **payload)
 
@@ -115,7 +115,7 @@ class TestAmphoraDriver(base.TestRpc):
             loadbalancer_id=self.sample_data.lb_id, name='Great LB')
         lb_dict = {'name': 'Great LB'}
         self.amp_driver.loadbalancer_update(old_provider_lb, provider_lb)
-        payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id,
+        payload = {consts.ORIGINAL_LOADBALANCER: old_provider_lb.to_dict(),
                    consts.LOAD_BALANCER_UPDATES: lb_dict}
         mock_cast.assert_called_with({}, 'update_load_balancer', **payload)
 
@@ -129,7 +129,7 @@ class TestAmphoraDriver(base.TestRpc):
             vip_qos_policy_id=qos_policy_id)
         lb_dict = {'vip': {'qos_policy_id': qos_policy_id}}
         self.amp_driver.loadbalancer_update(old_provider_lb, provider_lb)
-        payload = {consts.LOAD_BALANCER_ID: self.sample_data.lb_id,
+        payload = {consts.ORIGINAL_LOADBALANCER: old_provider_lb.to_dict(),
                    consts.LOAD_BALANCER_UPDATES: lb_dict}
         mock_cast.assert_called_with({}, 'update_load_balancer', **payload)
 
@@ -139,7 +139,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_listener = driver_dm.Listener(
             listener_id=self.sample_data.listener1_id)
         self.amp_driver.listener_create(provider_listener)
-        payload = {consts.LISTENER_ID: self.sample_data.listener1_id}
+        payload = {consts.LISTENER: provider_listener.to_dict()}
         mock_cast.assert_called_with({}, 'create_listener', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -147,7 +147,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_listener = driver_dm.Listener(
             listener_id=self.sample_data.listener1_id)
         self.amp_driver.listener_delete(provider_listener)
-        payload = {consts.LISTENER_ID: self.sample_data.listener1_id}
+        payload = {consts.LISTENER: provider_listener.to_dict()}
         mock_cast.assert_called_with({}, 'delete_listener', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -156,10 +156,11 @@ class TestAmphoraDriver(base.TestRpc):
             listener_id=self.sample_data.listener1_id)
         provider_listener = driver_dm.Listener(
             listener_id=self.sample_data.listener1_id, admin_state_up=False)
-        listener_dict = {'enabled': False}
+        listener_dict = provider_listener.to_dict()
+        listener_dict['admin_state_up'] = False
         self.amp_driver.listener_update(old_provider_listener,
                                         provider_listener)
-        payload = {consts.LISTENER_ID: self.sample_data.listener1_id,
+        payload = {consts.ORIGINAL_LISTENER: old_provider_listener.to_dict(),
                    consts.LISTENER_UPDATES: listener_dict}
         mock_cast.assert_called_with({}, 'update_listener', **payload)
 
@@ -169,10 +170,11 @@ class TestAmphoraDriver(base.TestRpc):
             listener_id=self.sample_data.listener1_id)
         provider_listener = driver_dm.Listener(
             listener_id=self.sample_data.listener1_id, name='Great Listener')
-        listener_dict = {'name': 'Great Listener'}
+        listener_dict = provider_listener.to_dict()
+        listener_dict['name'] = 'Great Listener'
         self.amp_driver.listener_update(old_provider_listener,
                                         provider_listener)
-        payload = {consts.LISTENER_ID: self.sample_data.listener1_id,
+        payload = {consts.ORIGINAL_LISTENER: old_provider_listener.to_dict(),
                    consts.LISTENER_UPDATES: listener_dict}
         mock_cast.assert_called_with({}, 'update_listener', **payload)
 
@@ -183,7 +185,7 @@ class TestAmphoraDriver(base.TestRpc):
             pool_id=self.sample_data.pool1_id,
             lb_algorithm=consts.LB_ALGORITHM_ROUND_ROBIN)
         self.amp_driver.pool_create(provider_pool)
-        payload = {consts.POOL_ID: self.sample_data.pool1_id}
+        payload = {consts.POOL: provider_pool.to_dict()}
         mock_cast.assert_called_with({}, 'create_pool', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -202,7 +204,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_pool = driver_dm.Pool(
             pool_id=self.sample_data.pool1_id)
         self.amp_driver.pool_delete(provider_pool)
-        payload = {consts.POOL_ID: self.sample_data.pool1_id}
+        payload = {consts.POOL: provider_pool.to_dict()}
         mock_cast.assert_called_with({}, 'delete_pool', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -224,7 +226,7 @@ class TestAmphoraDriver(base.TestRpc):
                      'ca_tls_certificate_id': 'CA REF',
                      'crl_container_id': 'CRL REF'}
         self.amp_driver.pool_update(old_provider_pool, provider_pool)
-        payload = {consts.POOL_ID: self.sample_data.pool1_id,
+        payload = {consts.ORIGINAL_POOL: old_provider_pool.to_dict(),
                    consts.POOL_UPDATES: pool_dict}
         mock_cast.assert_called_with({}, 'update_pool', **payload)
 
@@ -239,7 +241,7 @@ class TestAmphoraDriver(base.TestRpc):
                      'enabled': True,
                      'tls_enabled': True}
         self.amp_driver.pool_update(old_provider_pool, provider_pool)
-        payload = {consts.POOL_ID: self.sample_data.pool1_id,
+        payload = {consts.ORIGINAL_POOL: old_provider_pool.to_dict(),
                    consts.POOL_UPDATES: pool_dict}
         mock_cast.assert_called_with({}, 'update_pool', **payload)
 
@@ -263,9 +265,9 @@ class TestAmphoraDriver(base.TestRpc):
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_member_create(self, mock_cast, mock_pool_get, mock_session):
         provider_member = driver_dm.Member(
-            member_id=self.sample_data.member1_id)
+            member_id=self.sample_data)
         self.amp_driver.member_create(provider_member)
-        payload = {consts.MEMBER_ID: self.sample_data.member1_id}
+        payload = {consts.MEMBER: provider_member.to_dict()}
         mock_cast.assert_called_with({}, 'create_member', **payload)
 
     @mock.patch('octavia.db.api.get_session')
@@ -287,7 +289,7 @@ class TestAmphoraDriver(base.TestRpc):
             member_id=self.sample_data.member1_id,
             address="192.0.2.1")
         self.amp_driver.member_create(provider_member)
-        payload = {consts.MEMBER_ID: self.sample_data.member1_id}
+        payload = {consts.MEMBER: provider_member.to_dict()}
         mock_cast.assert_called_with({}, 'create_member', **payload)
 
     @mock.patch('octavia.db.api.get_session')
@@ -317,7 +319,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_member = driver_dm.Member(
             member_id=self.sample_data.member1_id)
         self.amp_driver.member_delete(provider_member)
-        payload = {consts.MEMBER_ID: self.sample_data.member1_id}
+        payload = {consts.MEMBER: provider_member.to_dict()}
         mock_cast.assert_called_with({}, 'delete_member', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -326,9 +328,12 @@ class TestAmphoraDriver(base.TestRpc):
             member_id=self.sample_data.member1_id)
         provider_member = driver_dm.Member(
             member_id=self.sample_data.member1_id, admin_state_up=True)
-        member_dict = {'enabled': True}
-        self.amp_driver.member_update(old_provider_member, provider_member)
-        payload = {consts.MEMBER_ID: self.sample_data.member1_id,
+        member_dict = provider_member.to_dict()
+        member_dict.pop(consts.MEMBER_ID)
+        member_dict['enabled'] = member_dict.pop('admin_state_up')
+        self.amp_driver.member_update(old_provider_member,
+                                      provider_member)
+        payload = {consts.ORIGINAL_MEMBER: old_provider_member.to_dict(),
                    consts.MEMBER_UPDATES: member_dict}
         mock_cast.assert_called_with({}, 'update_member', **payload)
 
@@ -338,9 +343,11 @@ class TestAmphoraDriver(base.TestRpc):
             member_id=self.sample_data.member1_id)
         provider_member = driver_dm.Member(
             member_id=self.sample_data.member1_id, name='Great member')
-        member_dict = {'name': 'Great member'}
-        self.amp_driver.member_update(old_provider_member, provider_member)
-        payload = {consts.MEMBER_ID: self.sample_data.member1_id,
+        member_dict = provider_member.to_dict()
+        member_dict.pop(consts.MEMBER_ID)
+        self.amp_driver.member_update(old_provider_member,
+                                      provider_member)
+        payload = {consts.ORIGINAL_MEMBER: old_provider_member.to_dict(),
                    consts.MEMBER_UPDATES: member_dict}
         mock_cast.assert_called_with({}, 'update_member', **payload)
 
@@ -372,11 +379,13 @@ class TestAmphoraDriver(base.TestRpc):
                            'protocol_port': 80,
                            'pool_id': self.sample_data.pool1_id}
 
-        self.amp_driver.member_batch_update(prov_members)
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, prov_members)
 
-        payload = {'old_member_ids': [self.sample_data.member1_id],
-                   'new_member_ids': [self.sample_data.member3_id],
-                   'updated_members': [update_mem_dict]}
+        payload = {
+            'old_members': [self.sample_data.db_pool1_members[0].to_dict()],
+            'new_members': [prov_new_member.to_dict()],
+            'updated_members': [update_mem_dict]}
         mock_cast.assert_called_with({}, 'batch_update_members', **payload)
 
     @mock.patch('octavia.db.api.get_session')
@@ -406,20 +415,35 @@ class TestAmphoraDriver(base.TestRpc):
                            'protocol_port': 80,
                            'pool_id': self.sample_data.pool1_id}
 
-        self.amp_driver.member_batch_update(prov_members)
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, prov_members)
 
-        payload = {'old_member_ids': [self.sample_data.member1_id],
-                   'new_member_ids': [self.sample_data.member3_id],
-                   'updated_members': [update_mem_dict]}
+        payload = {
+            'old_members': [self.sample_data.db_pool1_members[0].to_dict()],
+            'new_members': [prov_new_member.to_dict()],
+            'updated_members': [update_mem_dict]}
         mock_cast.assert_called_with({}, 'batch_update_members', **payload)
 
+    @mock.patch('octavia.db.api.get_session')
+    @mock.patch('octavia.db.repositories.PoolRepository.get')
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_member_batch_update_clear_already_empty(
+            self, mock_cast, mock_pool_get, mock_session):
+        mock_pool = mock.MagicMock()
+        mock_pool_get.return_value = mock_pool
+
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, [])
+
+        mock_cast.assert_not_called()
+
     # Health Monitor
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_health_monitor_create(self, mock_cast):
         provider_HM = driver_dm.HealthMonitor(
             healthmonitor_id=self.sample_data.hm1_id)
         self.amp_driver.health_monitor_create(provider_HM)
-        payload = {consts.HEALTH_MONITOR_ID: self.sample_data.hm1_id}
+        payload = {consts.HEALTH_MONITOR: provider_HM.to_dict()}
         mock_cast.assert_called_with({}, 'create_health_monitor', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -427,7 +451,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_HM = driver_dm.HealthMonitor(
             healthmonitor_id=self.sample_data.hm1_id)
         self.amp_driver.health_monitor_delete(provider_HM)
-        payload = {consts.HEALTH_MONITOR_ID: self.sample_data.hm1_id}
+        payload = {consts.HEALTH_MONITOR: provider_HM.to_dict()}
         mock_cast.assert_called_with({}, 'delete_health_monitor', **payload)
 
     @mock.patch('octavia.db.api.get_session')
@@ -467,10 +491,12 @@ class TestAmphoraDriver(base.TestRpc):
                            'protocol_port': 80,
                            'pool_id': self.sample_data.pool1_id}
 
-        self.amp_driver.member_batch_update(prov_members)
+        self.amp_driver.member_batch_update(
+            self.sample_data.pool1_id, prov_members)
 
-        payload = {'old_member_ids': [self.sample_data.member1_id],
-                   'new_member_ids': [self.sample_data.member3_id],
+        payload = {'old_members':
+                   [self.sample_data.db_pool1_members[0].to_dict()],
+                   'new_members': [prov_new_member.to_dict()],
                    'updated_members': [update_mem_dict]}
         mock_cast.assert_called_with({}, 'batch_update_members', **payload)
 
@@ -505,7 +531,7 @@ class TestAmphoraDriver(base.TestRpc):
 
         self.assertRaises(exceptions.UnsupportedOptionError,
                           self.amp_driver.member_batch_update,
-                          prov_members)
+                          self.sample_data.pool1_id, prov_members)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_health_monitor_update(self, mock_cast):
@@ -516,7 +542,7 @@ class TestAmphoraDriver(base.TestRpc):
             max_retries=1, max_retries_down=2)
         hm_dict = {'enabled': True, 'rise_threshold': 1, 'fall_threshold': 2}
         self.amp_driver.health_monitor_update(old_provider_hm, provider_hm)
-        payload = {consts.HEALTH_MONITOR_ID: self.sample_data.hm1_id,
+        payload = {consts.ORIGINAL_HEALTH_MONITOR: old_provider_hm.to_dict(),
                    consts.HEALTH_MONITOR_UPDATES: hm_dict}
         mock_cast.assert_called_with({}, 'update_health_monitor', **payload)
 
@@ -528,7 +554,7 @@ class TestAmphoraDriver(base.TestRpc):
             healthmonitor_id=self.sample_data.hm1_id, name='Great HM')
         hm_dict = {'name': 'Great HM'}
         self.amp_driver.health_monitor_update(old_provider_hm, provider_hm)
-        payload = {consts.HEALTH_MONITOR_ID: self.sample_data.hm1_id,
+        payload = {consts.ORIGINAL_HEALTH_MONITOR: old_provider_hm.to_dict(),
                    consts.HEALTH_MONITOR_UPDATES: hm_dict}
         mock_cast.assert_called_with({}, 'update_health_monitor', **payload)
 
@@ -538,7 +564,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_l7policy = driver_dm.L7Policy(
             l7policy_id=self.sample_data.l7policy1_id)
         self.amp_driver.l7policy_create(provider_l7policy)
-        payload = {consts.L7POLICY_ID: self.sample_data.l7policy1_id}
+        payload = {consts.L7POLICY: provider_l7policy.to_dict()}
         mock_cast.assert_called_with({}, 'create_l7policy', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -546,7 +572,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_l7policy = driver_dm.L7Policy(
             l7policy_id=self.sample_data.l7policy1_id)
         self.amp_driver.l7policy_delete(provider_l7policy)
-        payload = {consts.L7POLICY_ID: self.sample_data.l7policy1_id}
+        payload = {consts.L7POLICY: provider_l7policy.to_dict()}
         mock_cast.assert_called_with({}, 'delete_l7policy', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -558,7 +584,7 @@ class TestAmphoraDriver(base.TestRpc):
         l7policy_dict = {'enabled': True}
         self.amp_driver.l7policy_update(old_provider_l7policy,
                                         provider_l7policy)
-        payload = {consts.L7POLICY_ID: self.sample_data.l7policy1_id,
+        payload = {consts.ORIGINAL_L7POLICY: old_provider_l7policy.to_dict(),
                    consts.L7POLICY_UPDATES: l7policy_dict}
         mock_cast.assert_called_with({}, 'update_l7policy', **payload)
 
@@ -571,7 +597,7 @@ class TestAmphoraDriver(base.TestRpc):
         l7policy_dict = {'name': 'Great L7Policy'}
         self.amp_driver.l7policy_update(old_provider_l7policy,
                                         provider_l7policy)
-        payload = {consts.L7POLICY_ID: self.sample_data.l7policy1_id,
+        payload = {consts.ORIGINAL_L7POLICY: old_provider_l7policy.to_dict(),
                    consts.L7POLICY_UPDATES: l7policy_dict}
         mock_cast.assert_called_with({}, 'update_l7policy', **payload)
 
@@ -581,7 +607,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_l7rule = driver_dm.L7Rule(
             l7rule_id=self.sample_data.l7rule1_id)
         self.amp_driver.l7rule_create(provider_l7rule)
-        payload = {consts.L7RULE_ID: self.sample_data.l7rule1_id}
+        payload = {consts.L7RULE: provider_l7rule.to_dict()}
         mock_cast.assert_called_with({}, 'create_l7rule', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -589,7 +615,7 @@ class TestAmphoraDriver(base.TestRpc):
         provider_l7rule = driver_dm.L7Rule(
             l7rule_id=self.sample_data.l7rule1_id)
         self.amp_driver.l7rule_delete(provider_l7rule)
-        payload = {consts.L7RULE_ID: self.sample_data.l7rule1_id}
+        payload = {consts.L7RULE: provider_l7rule.to_dict()}
         mock_cast.assert_called_with({}, 'delete_l7rule', **payload)
 
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -599,8 +625,9 @@ class TestAmphoraDriver(base.TestRpc):
         provider_l7rule = driver_dm.L7Rule(
             l7rule_id=self.sample_data.l7rule1_id, admin_state_up=True)
         l7rule_dict = {'enabled': True}
-        self.amp_driver.l7rule_update(old_provider_l7rule, provider_l7rule)
-        payload = {consts.L7RULE_ID: self.sample_data.l7rule1_id,
+        self.amp_driver.l7rule_update(old_provider_l7rule,
+                                      provider_l7rule)
+        payload = {consts.ORIGINAL_L7RULE: old_provider_l7rule.to_dict(),
                    consts.L7RULE_UPDATES: l7rule_dict}
         mock_cast.assert_called_with({}, 'update_l7rule', **payload)
 
@@ -612,7 +639,7 @@ class TestAmphoraDriver(base.TestRpc):
             l7rule_id=self.sample_data.l7rule1_id, invert=True)
         l7rule_dict = {'invert': True}
         self.amp_driver.l7rule_update(old_provider_l7rule, provider_l7rule)
-        payload = {consts.L7RULE_ID: self.sample_data.l7rule1_id,
+        payload = {consts.ORIGINAL_L7RULE: old_provider_l7rule.to_dict(),
                    consts.L7RULE_UPDATES: l7rule_dict}
         mock_cast.assert_called_with({}, 'update_l7rule', **payload)
 
@@ -658,3 +685,90 @@ class TestAmphoraDriver(base.TestRpc):
                         'SUPPORTED_FLAVOR_SCHEMA', 'bogus'):
             self.assertRaises(exceptions.DriverError,
                               self.amp_driver.validate_flavor, 'bogus')
+
+    # Availability Zone
+    def test_get_supported_availability_zone_metadata(self):
+        test_schema = {
+            "properties": {
+                "test_name": {"description": "Test description"},
+                "test_name2": {"description": "Another description"}}}
+        ref_dict = {"test_name": "Test description",
+                    "test_name2": "Another description"}
+
+        # mock out the supported_availability_zone_metadata
+        with mock.patch('octavia.api.drivers.amphora_driver.'
+                        'availability_zone_schema.'
+                        'SUPPORTED_AVAILABILITY_ZONE_SCHEMA', test_schema):
+            result = self.amp_driver.get_supported_availability_zone_metadata()
+        self.assertEqual(ref_dict, result)
+
+        # Test for bad schema
+        with mock.patch('octavia.api.drivers.amphora_driver.'
+                        'availability_zone_schema.'
+                        'SUPPORTED_AVAILABILITY_ZONE_SCHEMA', 'bogus'):
+            self.assertRaises(
+                exceptions.DriverError,
+                self.amp_driver.get_supported_availability_zone_metadata)
+
+    def test_validate_availability_zone(self):
+        # Test compute zone
+        ref_dict = {consts.COMPUTE_ZONE: 'my_compute_zone'}
+        self.amp_driver.validate_availability_zone(ref_dict)
+
+        # Test vip networks
+        ref_dict = {consts.VALID_VIP_NETWORKS: ['my_vip_net']}
+        self.amp_driver.validate_availability_zone(ref_dict)
+
+        # Test management network
+        ref_dict = {consts.MANAGEMENT_NETWORK: 'my_management_net'}
+        self.amp_driver.validate_availability_zone(ref_dict)
+
+        # Test bad availability zone metadata key
+        ref_dict = {'bogus': 'bogus'}
+        self.assertRaises(exceptions.UnsupportedOptionError,
+                          self.amp_driver.validate_availability_zone,
+                          ref_dict)
+
+        # Test for bad schema
+        with mock.patch('octavia.api.drivers.amphora_driver.'
+                        'availability_zone_schema.'
+                        'SUPPORTED_AVAILABILITY_ZONE_SCHEMA', 'bogus'):
+            self.assertRaises(exceptions.DriverError,
+                              self.amp_driver.validate_availability_zone,
+                              'bogus')
+
+    @mock.patch('cryptography.fernet.Fernet')
+    def test_encrypt_listener_dict(self, mock_fernet):
+        mock_fern = mock.MagicMock()
+        mock_fernet.return_value = mock_fern
+        TEST_DATA = {'cert': b'some data'}
+        TEST_DATA2 = {'test': 'more data'}
+        FAKE_ENCRYPTED_DATA = b'alqwkhjetrhth'
+        mock_fern.encrypt.return_value = FAKE_ENCRYPTED_DATA
+
+        # We need a class instance with the mock
+        amp_driver = driver.AmphoraProviderDriver()
+
+        # Test just default_tls_container_data
+        list_dict = {consts.DEFAULT_TLS_CONTAINER_DATA: TEST_DATA}
+
+        amp_driver._encrypt_listener_dict(list_dict)
+
+        mock_fern.encrypt.assert_called_once_with(b'some data')
+
+        self.assertEqual({'cert': FAKE_ENCRYPTED_DATA},
+                         list_dict[consts.DEFAULT_TLS_CONTAINER_DATA])
+
+        mock_fern.reset_mock()
+
+        # Test just sni_container_data
+        TEST_DATA = {'cert': b'some data'}
+        sni_dict = {consts.SNI_CONTAINER_DATA: [TEST_DATA, TEST_DATA2]}
+
+        amp_driver._encrypt_listener_dict(sni_dict)
+
+        mock_fern.encrypt.assert_called_once_with(b'some data')
+
+        encrypted_sni = [{'cert': FAKE_ENCRYPTED_DATA},
+                         TEST_DATA2]
+        self.assertEqual(encrypted_sni, sni_dict[consts.SNI_CONTAINER_DATA])
