@@ -12,12 +12,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 #
+from unittest import mock
 
 from cryptography import fernet
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
+import tenacity
 
 from octavia.common import constants
 from octavia.common import exceptions
@@ -37,6 +38,7 @@ AMP_WAIT = 12
 AMPHORA_ID = uuidutils.generate_uuid()
 COMPUTE_ID = uuidutils.generate_uuid()
 LB_NET_IP = '192.0.2.1'
+LB_ID = uuidutils.generate_uuid()
 PORT_ID = uuidutils.generate_uuid()
 SERVER_GRPOUP_ID = uuidutils.generate_uuid()
 
@@ -50,11 +52,21 @@ class TestException(Exception):
         return repr(self.value)
 
 
-_amphora_mock = mock.MagicMock()
-_amphora_mock.id = AMPHORA_ID
-_amphora_mock.compute_id = COMPUTE_ID
-_load_balancer_mock = mock.MagicMock()
-_load_balancer_mock.amphorae = [_amphora_mock]
+_db_amphora_mock = mock.MagicMock()
+_db_amphora_mock.id = AMPHORA_ID
+_db_amphora_mock.compute_id = COMPUTE_ID
+_amphora_mock = {
+    constants.ID: AMPHORA_ID,
+    constants.COMPUTE_ID: COMPUTE_ID
+}
+_db_load_balancer_mock = mock.MagicMock()
+_db_load_balancer_mock.amphorae = [_db_amphora_mock]
+_db_load_balancer_mock.to_dict.return_value = {
+    constants.ID: LB_ID,
+}
+_load_balancer_mock = {
+    constants.LOADBALANCER_ID: LB_ID,
+}
 _port = mock.MagicMock()
 _port.id = PORT_ID
 
@@ -79,8 +91,8 @@ class TestComputeTasks(base.TestCase):
             group="controller_worker", amp_secgroup_list=AMP_SEC_GROUPS)
         self.conf.config(group="controller_worker", amp_image_owner_id='')
 
-        _amphora_mock.id = AMPHORA_ID
-        _amphora_mock.status = constants.AMPHORA_ALLOCATED
+        _db_amphora_mock.id = AMPHORA_ID
+        _db_amphora_mock.status = constants.AMPHORA_ALLOCATED
 
         logging_mock = mock.MagicMock()
         compute_tasks.LOG = logging_mock
@@ -106,12 +118,12 @@ class TestComputeTasks(base.TestCase):
 
         mock_driver.build.return_value = COMPUTE_ID
         # Test execute()
-        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port],
+        compute_id = createcompute.execute(_db_amphora_mock.id, ports=[_port],
                                            server_group_id=SERVER_GRPOUP_ID)
 
         # Validate that the build method was called properly
         mock_driver.build.assert_called_once_with(
-            name="amphora-" + _amphora_mock.id,
+            name="amphora-" + _db_amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
             image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
@@ -124,7 +136,9 @@ class TestComputeTasks(base.TestCase):
                                 'amphora-agent.conf': 'test_conf',
                                 '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG'},
             user_data=None,
-            server_group_id=SERVER_GRPOUP_ID)
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=None
+        )
 
         # Make sure it returns the expected compute_id
         self.assertEqual(COMPUTE_ID, compute_id)
@@ -134,14 +148,14 @@ class TestComputeTasks(base.TestCase):
 
         self.assertRaises(TypeError,
                           createcompute.execute,
-                          _amphora_mock, config_drive_files='test_cert')
+                          _db_amphora_mock, config_drive_files='test_cert')
 
         # Test revert()
 
-        _amphora_mock.compute_id = COMPUTE_ID
+        _db_amphora_mock.compute_id = COMPUTE_ID
 
         createcompute = compute_tasks.ComputeCreate()
-        createcompute.revert(compute_id, _amphora_mock.id)
+        createcompute.revert(compute_id, _db_amphora_mock.id)
 
         # Validate that the delete method was called properly
         mock_driver.delete.assert_called_once_with(
@@ -149,7 +163,7 @@ class TestComputeTasks(base.TestCase):
 
         # Test that a delete exception is not raised
 
-        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
+        createcompute.revert(COMPUTE_ID, _db_amphora_mock.id)
 
     @mock.patch('jinja2.Environment.get_template')
     @mock.patch('octavia.amphorae.backends.agent.'
@@ -169,11 +183,12 @@ class TestComputeTasks(base.TestCase):
 
         mock_driver.build.return_value = COMPUTE_ID
         # Test execute()
-        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port])
+        compute_id = createcompute.execute(_db_amphora_mock.id, ports=[_port],
+                                           server_group_id=None)
 
         # Validate that the build method was called properly
         mock_driver.build.assert_called_once_with(
-            name="amphora-" + _amphora_mock.id,
+            name="amphora-" + _db_amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
             image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
@@ -184,7 +199,8 @@ class TestComputeTasks(base.TestCase):
             port_ids=[PORT_ID],
             config_drive_files=None,
             user_data='test_ud_conf',
-            server_group_id=None)
+            server_group_id=None,
+            availability_zone=None)
 
         # Make sure it returns the expected compute_id
         self.assertEqual(COMPUTE_ID, compute_id)
@@ -194,14 +210,14 @@ class TestComputeTasks(base.TestCase):
 
         self.assertRaises(TypeError,
                           createcompute.execute,
-                          _amphora_mock, config_drive_files='test_cert')
+                          _db_amphora_mock, config_drive_files='test_cert')
 
         # Test revert()
 
-        _amphora_mock.compute_id = COMPUTE_ID
+        _db_amphora_mock.compute_id = COMPUTE_ID
 
         createcompute = compute_tasks.ComputeCreate()
-        createcompute.revert(compute_id, _amphora_mock.id)
+        createcompute.revert(compute_id, _db_amphora_mock.id)
 
         # Validate that the delete method was called properly
         mock_driver.delete.assert_called_once_with(
@@ -209,7 +225,76 @@ class TestComputeTasks(base.TestCase):
 
         # Test that a delete exception is not raised
 
-        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
+        createcompute.revert(COMPUTE_ID, _db_amphora_mock.id)
+
+    @mock.patch('octavia.common.jinja.logging.logging_jinja_cfg.'
+                'LoggingJinjaTemplater.build_logging_config')
+    @mock.patch('jinja2.Environment.get_template')
+    @mock.patch('octavia.amphorae.backends.agent.'
+                'agent_jinja_cfg.AgentJinjaTemplater.'
+                'build_agent_config', return_value='test_conf')
+    @mock.patch('stevedore.driver.DriverManager.driver')
+    def test_compute_create_availability_zone(self, mock_driver, mock_conf,
+                                              mock_jinja, mock_log_cfg):
+
+        image_owner_id = uuidutils.generate_uuid()
+        compute_zone = uuidutils.generate_uuid()
+        az_dict = {constants.COMPUTE_ZONE: compute_zone}
+
+        self.conf.config(
+            group="controller_worker", amp_image_owner_id=image_owner_id)
+        mock_log_cfg.return_value = 'FAKE CFG'
+
+        createcompute = compute_tasks.ComputeCreate()
+
+        mock_driver.build.return_value = COMPUTE_ID
+        # Test execute()
+        compute_id = createcompute.execute(_db_amphora_mock.id, ports=[_port],
+                                           server_group_id=SERVER_GRPOUP_ID,
+                                           availability_zone=az_dict)
+
+        # Validate that the build method was called properly
+        mock_driver.build.assert_called_once_with(
+            name="amphora-" + _db_amphora_mock.id,
+            amphora_flavor=AMP_FLAVOR_ID,
+            image_id=AMP_IMAGE_ID,
+            image_tag=AMP_IMAGE_TAG,
+            image_owner=image_owner_id,
+            key_name=AMP_SSH_KEY_NAME,
+            sec_groups=AMP_SEC_GROUPS,
+            network_ids=AMP_NET,
+            port_ids=[PORT_ID],
+            config_drive_files={'/etc/octavia/'
+                                'amphora-agent.conf': 'test_conf',
+                                '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG'},
+            user_data=None,
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=compute_zone)
+
+        # Make sure it returns the expected compute_id
+        self.assertEqual(COMPUTE_ID, compute_id)
+
+        # Test that a build exception is raised
+        createcompute = compute_tasks.ComputeCreate()
+
+        self.assertRaises(TypeError,
+                          createcompute.execute,
+                          _db_amphora_mock, config_drive_files='test_cert')
+
+        # Test revert()
+
+        _db_amphora_mock.compute_id = COMPUTE_ID
+
+        createcompute = compute_tasks.ComputeCreate()
+        createcompute.revert(compute_id, _db_amphora_mock.id)
+
+        # Validate that the delete method was called properly
+        mock_driver.delete.assert_called_once_with(
+            COMPUTE_ID)
+
+        # Test that a delete exception is not raised
+
+        createcompute.revert(COMPUTE_ID, _db_amphora_mock.id)
 
     @mock.patch('octavia.common.jinja.logging.logging_jinja_cfg.'
                 'LoggingJinjaTemplater.build_logging_config')
@@ -231,12 +316,12 @@ class TestComputeTasks(base.TestCase):
         mock_log_cfg.return_value = 'FAKE CFG'
 
         # Test execute()
-        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port],
+        compute_id = createcompute.execute(_db_amphora_mock.id, ports=[_port],
                                            server_group_id=SERVER_GRPOUP_ID)
 
         # Validate that the build method was called properly
         mock_driver.build.assert_called_once_with(
-            name="amphora-" + _amphora_mock.id,
+            name="amphora-" + _db_amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
             image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
@@ -249,7 +334,8 @@ class TestComputeTasks(base.TestCase):
                                 'amphora-agent.conf': 'test_conf',
                                 '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG'},
             user_data=None,
-            server_group_id=SERVER_GRPOUP_ID)
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=None)
 
         self.assertEqual(COMPUTE_ID, compute_id)
 
@@ -258,14 +344,14 @@ class TestComputeTasks(base.TestCase):
 
         self.assertRaises(TypeError,
                           createcompute.execute,
-                          _amphora_mock, config_drive_files='test_cert')
+                          _db_amphora_mock, config_drive_files='test_cert')
 
         # Test revert()
 
-        _amphora_mock.compute_id = COMPUTE_ID
+        _db_amphora_mock.compute_id = COMPUTE_ID
 
         createcompute = compute_tasks.ComputeCreate()
-        createcompute.revert(compute_id, _amphora_mock.id)
+        createcompute.revert(compute_id, _db_amphora_mock.id)
 
         # Validate that the delete method was called properly
         mock_driver.delete.assert_called_once_with(
@@ -273,7 +359,7 @@ class TestComputeTasks(base.TestCase):
 
         # Test that a delete exception is not raised
 
-        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
+        createcompute.revert(COMPUTE_ID, _db_amphora_mock.id)
 
     @mock.patch('octavia.common.jinja.logging.logging_jinja_cfg.'
                 'LoggingJinjaTemplater.build_logging_config')
@@ -285,7 +371,7 @@ class TestComputeTasks(base.TestCase):
     def test_compute_create_cert(self, mock_driver, mock_conf, mock_jinja,
                                  mock_log_cfg):
         createcompute = compute_tasks.CertComputeCreate()
-        key = utils.get_six_compatible_server_certs_key_passphrase()
+        key = utils.get_compatible_server_certs_key_passphrase()
         fer = fernet.Fernet(key)
         mock_log_cfg.return_value = 'FAKE CFG'
 
@@ -294,15 +380,15 @@ class TestComputeTasks(base.TestCase):
         self.useFixture(test_utils.OpenFixture(path, 'test'))
         # Test execute()
         test_cert = fer.encrypt(
-            utils.get_six_compatible_value('test_cert')
-        )
-        compute_id = createcompute.execute(_amphora_mock.id, test_cert,
+            utils.get_compatible_value('test_cert')
+        ).decode('utf-8')
+        compute_id = createcompute.execute(_db_amphora_mock.id, test_cert,
                                            server_group_id=SERVER_GRPOUP_ID
                                            )
 
         # Validate that the build method was called properly
         mock_driver.build.assert_called_once_with(
-            name="amphora-" + _amphora_mock.id,
+            name="amphora-" + _db_amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
             image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
@@ -314,10 +400,12 @@ class TestComputeTasks(base.TestCase):
             user_data=None,
             config_drive_files={
                 '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG',
-                '/etc/octavia/certs/server.pem': fer.decrypt(test_cert),
+                '/etc/octavia/certs/server.pem': fer.decrypt(
+                    test_cert.encode('utf-8')),
                 '/etc/octavia/certs/client_ca.pem': 'test',
                 '/etc/octavia/amphora-agent.conf': 'test_conf'},
-            server_group_id=SERVER_GRPOUP_ID)
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=None)
 
         self.assertEqual(COMPUTE_ID, compute_id)
 
@@ -327,22 +415,22 @@ class TestComputeTasks(base.TestCase):
         createcompute = compute_tasks.ComputeCreate()
         self.assertRaises(TypeError,
                           createcompute.execute,
-                          _amphora_mock,
+                          _db_amphora_mock,
                           config_drive_files=test_cert)
 
         # Test revert()
 
-        _amphora_mock.compute_id = COMPUTE_ID
+        _db_amphora_mock.compute_id = COMPUTE_ID
 
         createcompute = compute_tasks.ComputeCreate()
-        createcompute.revert(compute_id, _amphora_mock.id)
+        createcompute.revert(compute_id, _db_amphora_mock.id)
 
         # Validate that the delete method was called properly
         mock_driver.delete.assert_called_once_with(COMPUTE_ID)
 
         # Test that a delete exception is not raised
 
-        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
+        createcompute.revert(COMPUTE_ID, _db_amphora_mock.id)
 
     @mock.patch('octavia.controller.worker.amphora_rate_limit'
                 '.AmphoraBuildRateLimit.remove_from_build_req_queue')
@@ -354,22 +442,30 @@ class TestComputeTasks(base.TestCase):
                           mock_remove_from_build_queue):
 
         self.conf.config(group='haproxy_amphora', build_rate_limit=5)
-        _amphora_mock.compute_id = COMPUTE_ID
-        _amphora_mock.status = constants.ACTIVE
-        _amphora_mock.lb_network_ip = LB_NET_IP
+        _db_amphora_mock.compute_id = COMPUTE_ID
+        _db_amphora_mock.status = constants.ACTIVE
+        _db_amphora_mock.lb_network_ip = LB_NET_IP
 
-        mock_driver.get_amphora.return_value = _amphora_mock, None
+        mock_driver.get_amphora.return_value = _db_amphora_mock, None
 
         computewait = compute_tasks.ComputeActiveWait()
-        computewait.execute(COMPUTE_ID, AMPHORA_ID)
 
-        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
+        # Test with no AZ
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, None)
+        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID, None)
 
-        _amphora_mock.status = constants.DELETED
+        # Test with AZ
+        mock_driver.reset_mock()
+        az = {constants.MANAGEMENT_NETWORK: uuidutils.generate_uuid()}
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, az)
+        mock_driver.get_amphora.assert_called_once_with(
+            COMPUTE_ID, az[constants.MANAGEMENT_NETWORK])
 
+        # Test with deleted amp
+        _db_amphora_mock.status = constants.DELETED
         self.assertRaises(exceptions.ComputeWaitTimeoutException,
                           computewait.execute,
-                          _amphora_mock, AMPHORA_ID)
+                          _amphora_mock, AMPHORA_ID, None)
 
     @mock.patch('octavia.controller.worker.amphora_rate_limit'
                 '.AmphoraBuildRateLimit.remove_from_build_req_queue')
@@ -381,22 +477,22 @@ class TestComputeTasks(base.TestCase):
                                        mock_remove_from_build_queue):
 
         self.conf.config(group='haproxy_amphora', build_rate_limit=5)
-        _amphora_mock.compute_id = COMPUTE_ID
-        _amphora_mock.status = constants.ACTIVE
-        _amphora_mock.lb_network_ip = LB_NET_IP
+        _db_amphora_mock.compute_id = COMPUTE_ID
+        _db_amphora_mock.status = constants.ACTIVE
+        _db_amphora_mock.lb_network_ip = LB_NET_IP
 
-        mock_driver.get_amphora.return_value = _amphora_mock, None
+        mock_driver.get_amphora.return_value = _db_amphora_mock, None
 
         computewait = compute_tasks.ComputeActiveWait()
-        computewait.execute(COMPUTE_ID, AMPHORA_ID)
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, None)
 
-        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
+        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID, None)
 
-        _amphora_mock.status = constants.ERROR
+        _db_amphora_mock.status = constants.ERROR
 
         self.assertRaises(exceptions.ComputeBuildException,
                           computewait.execute,
-                          _amphora_mock, AMPHORA_ID)
+                          _db_amphora_mock, AMPHORA_ID, None)
 
     @mock.patch('octavia.controller.worker.amphora_rate_limit'
                 '.AmphoraBuildRateLimit.remove_from_build_req_queue')
@@ -406,34 +502,72 @@ class TestComputeTasks(base.TestCase):
                                   mock_time_sleep,
                                   mock_driver,
                                   mock_remove_from_build_queue):
-        _amphora_mock.compute_id = COMPUTE_ID
-        _amphora_mock.status = constants.ACTIVE
-        _amphora_mock.lb_network_ip = LB_NET_IP
+        _db_amphora_mock.compute_id = COMPUTE_ID
+        _db_amphora_mock.status = constants.ACTIVE
+        _db_amphora_mock.lb_network_ip = LB_NET_IP
 
-        mock_driver.get_amphora.return_value = _amphora_mock, None
+        mock_driver.get_amphora.return_value = _db_amphora_mock, None
 
         computewait = compute_tasks.ComputeActiveWait()
-        computewait.execute(COMPUTE_ID, AMPHORA_ID)
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, None)
 
-        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
+        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID, None)
         mock_remove_from_build_queue.assert_not_called()
 
     @mock.patch('stevedore.driver.DriverManager.driver')
-    def test_delete_amphorae_on_load_balancer(self, mock_driver):
+    @mock.patch('octavia.db.api.get_session', return_value=mock.MagicMock())
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
+    def test_delete_amphorae_on_load_balancer(self, mock_lb_get, mock_session,
+                                              mock_driver):
 
+        mock_driver.delete.side_effect = [mock.DEFAULT,
+                                          exceptions.OctaviaException('boom')]
         delete_amps = compute_tasks.DeleteAmphoraeOnLoadBalancer()
+
+        mock_lb_get.return_value = _db_load_balancer_mock
         delete_amps.execute(_load_balancer_mock)
 
         mock_driver.delete.assert_called_once_with(COMPUTE_ID)
 
+        # Test compute driver exception is raised
+        self.assertRaises(exceptions.OctaviaException, delete_amps.execute,
+                          _load_balancer_mock)
+
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_compute_delete(self, mock_driver):
+        mock_driver.delete.side_effect = [
+            mock.DEFAULT, exceptions.OctaviaException('boom'),
+            mock.DEFAULT, exceptions.OctaviaException('boom'),
+            exceptions.OctaviaException('boom'),
+            exceptions.OctaviaException('boom'),
+            exceptions.OctaviaException('boom')]
 
         delete_compute = compute_tasks.ComputeDelete()
+
+        # Limit the retry attempts for the test run to save time
+        delete_compute.execute.retry.stop = tenacity.stop_after_attempt(2)
+
         delete_compute.execute(_amphora_mock)
 
         mock_driver.delete.assert_called_once_with(COMPUTE_ID)
 
+        # Test retry after a compute exception
+        mock_driver.reset_mock()
+        delete_compute.execute(_amphora_mock)
+        mock_driver.delete.assert_has_calls([mock.call(COMPUTE_ID),
+                                            mock.call(COMPUTE_ID)])
+
+        # Test passive failure
+        mock_driver.reset_mock()
+        delete_compute.execute(_amphora_mock, passive_failure=True)
+        mock_driver.delete.assert_has_calls([mock.call(COMPUTE_ID),
+                                            mock.call(COMPUTE_ID)])
+
+        # Test non-passive failure
+        mock_driver.reset_mock()
+        self.assertRaises(exceptions.OctaviaException, delete_compute.execute,
+                          _amphora_mock, passive_failure=False)
+
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_nova_server_group_create(self, mock_driver):
         nova_sever_group_obj = compute_tasks.NovaServerGroupCreate()
@@ -479,3 +613,32 @@ class TestComputeTasks(base.TestCase):
         sg_id = None
         nova_sever_group_obj.execute(sg_id)
         self.assertFalse(mock_driver.delete_server_group.called, sg_id)
+
+    @mock.patch('stevedore.driver.DriverManager.driver')
+    def test_attach_port(self, mock_driver):
+        COMPUTE_ID = uuidutils.generate_uuid()
+        PORT_ID = uuidutils.generate_uuid()
+        amphora_dict = {constants.COMPUTE_ID: COMPUTE_ID}
+        port_dict = {constants.ID: PORT_ID}
+
+        attach_port_obj = compute_tasks.AttachPort()
+
+        # Test execute
+        attach_port_obj.execute(amphora_dict, port_dict)
+
+        mock_driver.attach_network_or_port.assert_called_once_with(
+            COMPUTE_ID, port_id=PORT_ID)
+
+        # Test revert
+        mock_driver.reset_mock()
+
+        attach_port_obj.revert(amphora_dict, port_dict)
+
+        mock_driver.detach_port.assert_called_once_with(COMPUTE_ID, PORT_ID)
+
+        # Test rever exception
+        mock_driver.reset_mock()
+        mock_driver.detach_port.side_effect = [Exception('boom')]
+
+        # should not raise
+        attach_port_obj.revert(amphora_dict, port_dict)
